# -*- coding: utf-8 -*-
"""QeBaderWorkChain workchain of the AiiDA bader plugin"""

from __future__ import absolute_import

from aiida.common import AttributeDict
from aiida.engine import ToContext, WorkChain
from aiida.plugins import CalculationFactory, WorkflowFactory


PwBaseWorkChain = WorkflowFactory(
    "quantumespresso.pw.base"
)  # pylint: disable=invalid-name
PpCalculation = CalculationFactory("quantumespresso.pp")  # pylint: disable=invalid-name
BaderCalculation = CalculationFactory("bader")  # pylint: disable=invalid-name


class QeBaderWorkChain(WorkChain):
    """A workchain that computes bader charges using QE and Bader code."""

    @classmethod
    def define(cls, spec):
        """Define workflow specification."""
        super(QeBaderWorkChain, cls).define(spec)

        spec.expose_inputs(PwBaseWorkChain, namespace="pw_base")
        spec.expose_inputs(PpCalculation, namespace="pp", exclude=["parent_folder"])
        spec.expose_inputs(
            BaderCalculation, namespace="bader", exclude=["charge_density_folder"]
        )

        spec.outline(cls.run_pw, cls.run_pp, cls.run_bader, cls.return_results)

        spec.expose_outputs(PwBaseWorkChain, namespace="pw_base")
        spec.expose_outputs(PpCalculation, namespace="pp")
        spec.expose_outputs(BaderCalculation, namespace="bader")

        spec.exit_code(903, "ERROR_PARSING_PW_OUTPUT", "Error while parsing PW output")
        spec.exit_code(904, "ERROR_PARSING_PP_OUTPUT", "Error while parsing PP output")
        spec.exit_code(
            905, "ERROR_PARSING_BADER_OUTPUT", "Error while parsing bader output"
        )

    def run_pw(self):
        """Run PW."""
        pw_base_inputs = AttributeDict(self.exposed_inputs(PwBaseWorkChain, "pw_base"))
        pw_base_inputs["metadata"]["label"] = "pw_scf"
        pw_base_inputs["metadata"]["call_link_label"] = "call_pw_scf"
        running = self.submit(PwBaseWorkChain, **pw_base_inputs)
        self.report("Running PwBaseWorkChain.")
        return ToContext(pw_calc=running)

    def run_pp(self):
        """Run PP to generate the charge-density."""
        # TODO extract number of core electrons from the pw pseudopotential

        try:
            pp_inputs = AttributeDict(self.exposed_inputs(PpCalculation, "pp"))
            pp_inputs["parent_folder"] = self.ctx.pw_calc.outputs.remote_folder
        except Exception as exc:  # pylint: disable=broad-except
            self.report(f'Encountered exception "{str(exc)}" while parsing PW output')
            return self.exit_codes.ERROR_PARSING_PW_OUTPUT  # pylint: disable=no-member

        pp_inputs["metadata"]["call_link_label"] = "call_pp_calc"

        # Create the calculation process and launch it
        running = self.submit(PpCalculation, **pp_inputs)
        self.report(
            f"Running PpCalculation<{running.pk}> to compute the charge-density"
        )
        return ToContext(pp_calc=running)

    def run_bader(self):
        """Parse the PP ouputs cube file, and submit bader calculation."""
        try:
            bader_inputs = AttributeDict(self.exposed_inputs(BaderCalculation, "bader"))
            bader_inputs[
                "charge_density_folder"
            ] = self.ctx.pp_calc.outputs.remote_folder
        except Exception as exc:  # pylint: disable=broad-except
            self.report(f'Encountered exception "{str(exc)}" while parsing PP output')
            return self.exit_codes.ERROR_PARSING_PP_OUTPUT  # pylint: disable=no-member

        bader_inputs["metadata"]["call_link_label"] = "call_bader_calc"

        # Create the calculation process and launch it
        running = self.submit(BaderCalculation, **bader_inputs)
        self.report(
            f"Running BaderCalculation<{running.pk}> to compute point charges from the charge-density"
        )
        return ToContext(bader_calc=running)

    def return_results(self):
        """Return exposed outputs and print the pk of the ArrayData w/bader"""
        try:
            self.out_many(
                self.exposed_outputs(
                    self.ctx.pw_calc, PwBaseWorkChain, namespace="pw_base"
                )
            )
            self.out_many(
                self.exposed_outputs(self.ctx.pp_calc, PpCalculation, namespace="pp")
            )
            self.out_many(
                self.exposed_outputs(
                    self.ctx.bader_calc, BaderCalculation, namespace="bader"
                )
            )
            self.report(
                f"bader charges computed: ArrayData<{self.outputs['bader']['bader_charge'].pk}>"
            )
        except KeyError:
            return (
                self.exit_codes.ERROR_PARSING_BADER_OUTPUT
            )  # pylint: disable=no-member

        return 0



# -*- coding: utf-8 -*-
"""Cp2kBaderWorkTree of the AiiDA bader plugin"""

from aiida_worktree import build_node, WorkTree


def Cp2kBaderWorkTree():
    # register node
    cp2k_node = build_node({"path": "aiida_cp2k.workchains.base.Cp2kBaseWorkChain"})
    bader_node = build_node({"path": "aiida_bader.calculations.BaderCalculation"})
    # create worktree
    wt = WorkTree("charge-density")
    cp2k_node = wt.nodes.new(cp2k_node, name="cp2k_base")
    bader_node = wt.nodes.new(bader_node, name="bader")
    wt.links.new(
        cp2k_node.outputs["remote_folder"], bader_node.inputs["charge_density_folder"]
    )
    return wt



import pathlib
from setuptools import setup, find_packages


def test_suite():
    import unittest

    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover("tests", pattern="test_*.py")
    return test_suite


# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name="aiida-bader",
    version="0.0.1",
    description="AiiDA plugin for bader code.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/superstart54/aiida-bader",
    author="Xing Wang",
    author_email="xingwang1991@gmail.com",
    license="MIT License",
    classifiers=[],
    packages=find_packages(),
    install_requires=[
        "aiida-core",
        "aiida-worktree",
        "aiida-quantumespresso",
        "aiida-cp2k",
        "pytest",
        "pytest-cov",
        "pre-commit",
    ],
    entry_points={
        "aiida.calculations": [
            "bader = aiida_bader.calculations:BaderCalculation",
        ],
        "aiida.parsers": [
            "bader = aiida_bader.parsers:BaderParser",
        ],
        "aiida.workflows": [
            "bader.qe = aiida_bader.workchains:QeBaderWorkChain",
        ],
    },
    package_data={},
    python_requires=">=3.8",
    test_suite="setup.test_suite",
)



# -*- coding: utf-8 -*-
"""QeBaderWorkTree of the AiiDA bader plugin"""

from aiida_worktree import build_node, WorkTree


def QeBaderWorkTree():
    # register node
    pw_node = build_node(
        {"path": "aiida_quantumespresso.workflows.pw.base.PwBaseWorkChain"}
    )
    pp_node = build_node(
        {"path": "aiida_quantumespresso.calculations.pp.PpCalculation"}
    )
    bader_node = build_node({"path": "aiida_bader.calculations.BaderCalculation"})
    # create worktree
    wt = WorkTree("charge-density")
    pw_node = wt.nodes.new(pw_node, name="pw_base")
    pp_node = wt.nodes.new(pp_node, name="pp")
    bader_node = wt.nodes.new(bader_node, name="bader")
    wt.links.new(pw_node.outputs["remote_folder"], pp_node.inputs["parent_folder"])
    wt.links.new(
        pp_node.outputs["remote_folder"], bader_node.inputs["charge_density_folder"]
    )
    # export worktree
    return wt
